from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Item(__BaseDictObject):
    """
    A listings item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "sku" in data:
            self.sku: str = self._get_value(str, "sku")
        else:
            self.sku: str = None
        if "summaries" in data:
            self.summaries: ItemSummaries = self._get_value(ItemSummaries, "summaries")
        else:
            self.summaries: ItemSummaries = None
        if "attributes" in data:
            self.attributes: ItemAttributes = self._get_value(ItemAttributes, "attributes")
        else:
            self.attributes: ItemAttributes = None
        if "issues" in data:
            self.issues: ItemIssues = self._get_value(ItemIssues, "issues")
        else:
            self.issues: ItemIssues = None
        if "offers" in data:
            self.offers: ItemOffers = self._get_value(ItemOffers, "offers")
        else:
            self.offers: ItemOffers = None
        if "fulfillmentAvailability" in data:
            self.fulfillmentAvailability: _List[FulfillmentAvailability] = [
                FulfillmentAvailability(datum) for datum in data["fulfillmentAvailability"]
            ]
        else:
            self.fulfillmentAvailability: _List[FulfillmentAvailability] = []
        if "procurement" in data:
            self.procurement: ItemProcurement = self._get_value(ItemProcurement, "procurement")
        else:
            self.procurement: ItemProcurement = None


class ItemSummaryByMarketplace(__BaseDictObject):
    """
    Summary details of a listings item for an Amazon marketplace.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceId" in data:
            self.marketplaceId: str = self._get_value(str, "marketplaceId")
        else:
            self.marketplaceId: str = None
        if "asin" in data:
            self.asin: str = self._get_value(str, "asin")
        else:
            self.asin: str = None
        if "productType" in data:
            self.productType: str = self._get_value(str, "productType")
        else:
            self.productType: str = None
        if "conditionType" in data:
            self.conditionType: str = self._get_value(str, "conditionType")
        else:
            self.conditionType: str = None
        if "status" in data:
            self.status: _List[str] = [str(datum) for datum in data["status"]]
        else:
            self.status: _List[str] = []
        if "fnSku" in data:
            self.fnSku: str = self._get_value(str, "fnSku")
        else:
            self.fnSku: str = None
        if "itemName" in data:
            self.itemName: str = self._get_value(str, "itemName")
        else:
            self.itemName: str = None
        if "createdDate" in data:
            self.createdDate: str = self._get_value(str, "createdDate")
        else:
            self.createdDate: str = None
        if "lastUpdatedDate" in data:
            self.lastUpdatedDate: str = self._get_value(str, "lastUpdatedDate")
        else:
            self.lastUpdatedDate: str = None
        if "mainImage" in data:
            self.mainImage: ItemImage = self._get_value(ItemImage, "mainImage")
        else:
            self.mainImage: ItemImage = None


class ItemImage(__BaseDictObject):
    """
    Image for the listings item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "link" in data:
            self.link: str = self._get_value(str, "link")
        else:
            self.link: str = None
        if "height" in data:
            self.height: int = self._get_value(int, "height")
        else:
            self.height: int = None
        if "width" in data:
            self.width: int = self._get_value(int, "width")
        else:
            self.width: int = None


class ItemAttributes(__BaseDictObject):
    """
    JSON object containing structured listings item attribute data keyed by attribute name.
    """

    def __init__(self, data):
        super().__init__(data)


class Issue(__BaseDictObject):
    """
    An issue with a listings item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "severity" in data:
            self.severity: str = self._get_value(str, "severity")
        else:
            self.severity: str = None
        if "attributeNames" in data:
            self.attributeNames: _List[str] = [str(datum) for datum in data["attributeNames"]]
        else:
            self.attributeNames: _List[str] = []


class ItemOfferByMarketplace(__BaseDictObject):
    """
    Offer details of a listings item for an Amazon marketplace.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceId" in data:
            self.marketplaceId: str = self._get_value(str, "marketplaceId")
        else:
            self.marketplaceId: str = None
        if "offerType" in data:
            self.offerType: str = self._get_value(str, "offerType")
        else:
            self.offerType: str = None
        if "price" in data:
            self.price: Money = self._get_value(Money, "price")
        else:
            self.price: Money = None
        if "points" in data:
            self.points: Points = self._get_value(Points, "points")
        else:
            self.points: Points = None


class ItemProcurement(__BaseDictObject):
    """
    Vendor procurement information for the listings item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "costPrice" in data:
            self.costPrice: Money = self._get_value(Money, "costPrice")
        else:
            self.costPrice: Money = None


class FulfillmentAvailability(__BaseDictObject):
    """
    Fulfillment availability details for the listings item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "fulfillmentChannelCode" in data:
            self.fulfillmentChannelCode: str = self._get_value(str, "fulfillmentChannelCode")
        else:
            self.fulfillmentChannelCode: str = None
        if "quantity" in data:
            self.quantity: int = self._get_value(int, "quantity")
        else:
            self.quantity: int = None


class Money(__BaseDictObject):
    """
    The currency type and the amount.
    """

    def __init__(self, data):
        super().__init__(data)
        if "currencyCode" in data:
            self.currencyCode: str = self._get_value(str, "currencyCode")
        else:
            self.currencyCode: str = None
        if "amount" in data:
            self.amount: Decimal = self._get_value(Decimal, "amount")
        else:
            self.amount: Decimal = None


class Points(__BaseDictObject):
    """
    The number of Amazon Points offered with the purchase of an item, and their monetary value. Note that the Points element is only returned in Japan (JP).
    """

    def __init__(self, data):
        super().__init__(data)
        if "pointsNumber" in data:
            self.pointsNumber: int = self._get_value(int, "pointsNumber")
        else:
            self.pointsNumber: int = None


class PatchOperation(__BaseDictObject):
    """
    Individual JSON Patch operation for an HTTP PATCH request.
    """

    def __init__(self, data):
        super().__init__(data)
        if "op" in data:
            self.op: str = self._get_value(str, "op")
        else:
            self.op: str = None
        if "path" in data:
            self.path: str = self._get_value(str, "path")
        else:
            self.path: str = None
        if "value" in data:
            self.value: _List[dict] = [dict(datum) for datum in data["value"]]
        else:
            self.value: _List[dict] = []


class ListingsItemPatchRequest(__BaseDictObject):
    """
    The request body schema for the patchListingsItem operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "productType" in data:
            self.productType: str = self._get_value(str, "productType")
        else:
            self.productType: str = None
        if "patches" in data:
            self.patches: _List[PatchOperation] = [PatchOperation(datum) for datum in data["patches"]]
        else:
            self.patches: _List[PatchOperation] = []


class ListingsItemPutRequest(__BaseDictObject):
    """
    The request body schema for the putListingsItem operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "productType" in data:
            self.productType: str = self._get_value(str, "productType")
        else:
            self.productType: str = None
        if "requirements" in data:
            self.requirements: str = self._get_value(str, "requirements")
        else:
            self.requirements: str = None
        if "attributes" in data:
            self.attributes: dict = self._get_value(dict, "attributes")
        else:
            self.attributes: dict = None


class ListingsItemSubmissionResponse(__BaseDictObject):
    """
    Response containing the results of a submission to the Selling Partner API for Listings Items.
    """

    def __init__(self, data):
        super().__init__(data)
        if "sku" in data:
            self.sku: str = self._get_value(str, "sku")
        else:
            self.sku: str = None
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "submissionId" in data:
            self.submissionId: str = self._get_value(str, "submissionId")
        else:
            self.submissionId: str = None
        if "issues" in data:
            self.issues: _List[Issue] = [Issue(datum) for datum in data["issues"]]
        else:
            self.issues: _List[Issue] = []


class ItemSummaries(list, _List["ItemSummaryByMarketplace"]):
    """
    Summary details of a listings item.
    """

    def __init__(self, data):
        super().__init__([ItemSummaryByMarketplace(datum) for datum in data])
        self.data = data


class ItemIssues(list, _List["Issue"]):
    """
    Issues associated with the listings item.
    """

    def __init__(self, data):
        super().__init__([Issue(datum) for datum in data])
        self.data = data


class ItemOffers(list, _List["ItemOfferByMarketplace"]):
    """
    Offer details for the listings item.
    """

    def __init__(self, data):
        super().__init__([ItemOfferByMarketplace(datum) for datum in data])
        self.data = data


class Decimal(str):
    """
    A decimal number with no loss of precision. Useful when precision loss is unnaceptable, as with currencies. Follows RFC7159 for number representation.
    """


class ListingsItems20210801Client(__BaseClient):
    def getListingsItem(
        self,
        sellerId: str,
        sku: str,
        marketplaceIds: _List[str],
        issueLocale: str = None,
        includedData: _List[str] = None,
    ):
        """
                Returns details about a listings item for a selling partner.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 5 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/listings/2021-08-01/items/{sellerId}/{sku}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if issueLocale is not None:
            params["issueLocale"] = issueLocale
        if includedData is not None:
            params["includedData"] = ",".join(map(str, includedData))
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: Item,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def putListingsItem(
        self,
        data: ListingsItemPutRequest,
        sellerId: str,
        sku: str,
        marketplaceIds: _List[str],
        issueLocale: str = None,
    ):
        """
                Creates a new or fully-updates an existing listings item for a selling partner.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 5 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/listings/2021-08-01/items/{sellerId}/{sku}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if issueLocale is not None:
            params["issueLocale"] = issueLocale
        response = self.request(
            path=url,
            method="PUT",
            params=params,
            data=data.data,
        )
        response_type = {
            200: ListingsItemSubmissionResponse,
            400: ErrorList,
            403: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def deleteListingsItem(
        self,
        sellerId: str,
        sku: str,
        marketplaceIds: _List[str],
        issueLocale: str = None,
    ):
        """
                Delete a listings item for a selling partner.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 5 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/listings/2021-08-01/items/{sellerId}/{sku}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if issueLocale is not None:
            params["issueLocale"] = issueLocale
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: ListingsItemSubmissionResponse,
            400: ErrorList,
            403: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def patchListingsItem(
        self,
        data: ListingsItemPatchRequest,
        sellerId: str,
        sku: str,
        marketplaceIds: _List[str],
        issueLocale: str = None,
    ):
        """
                Partially update (patch) a listings item for a selling partner. Only top-level listings item attributes can be patched. Patching nested attributes is not supported.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 5 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/listings/2021-08-01/items/{sellerId}/{sku}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if issueLocale is not None:
            params["issueLocale"] = issueLocale
        response = self.request(
            path=url,
            method="PATCH",
            params=params,
            data=data.data,
        )
        response_type = {
            200: ListingsItemSubmissionResponse,
            400: ErrorList,
            403: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class GetPricingResponse(__BaseDictObject):
    """
    The response schema for the getPricing and getCompetitivePricing operations.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: PriceList = self._get_value(PriceList, "payload")
        else:
            self.payload: PriceList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOffersResponse(__BaseDictObject):
    """
    The response schema for the getListingOffers and getItemOffers operations.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: GetOffersResult = self._get_value(GetOffersResult, "payload")
        else:
            self.payload: GetOffersResult = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOffersResult(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceID" in data:
            self.MarketplaceID: str = self._get_value(str, "MarketplaceID")
        else:
            self.MarketplaceID: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "SKU" in data:
            self.SKU: str = self._get_value(str, "SKU")
        else:
            self.SKU: str = None
        if "ItemCondition" in data:
            self.ItemCondition: ConditionType = self._get_value(ConditionType, "ItemCondition")
        else:
            self.ItemCondition: ConditionType = None
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "Identifier" in data:
            self.Identifier: ItemIdentifier = self._get_value(ItemIdentifier, "Identifier")
        else:
            self.Identifier: ItemIdentifier = None
        if "Summary" in data:
            self.Summary: Summary = self._get_value(Summary, "Summary")
        else:
            self.Summary: Summary = None
        if "Offers" in data:
            self.Offers: OfferDetailList = self._get_value(OfferDetailList, "Offers")
        else:
            self.Offers: OfferDetailList = None


class Price(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "Product" in data:
            self.Product: Product = self._get_value(Product, "Product")
        else:
            self.Product: Product = None


class Product(__BaseDictObject):
    """
    An item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Identifiers" in data:
            self.Identifiers: IdentifierType = self._get_value(IdentifierType, "Identifiers")
        else:
            self.Identifiers: IdentifierType = None
        if "AttributeSets" in data:
            self.AttributeSets: AttributeSetList = self._get_value(AttributeSetList, "AttributeSets")
        else:
            self.AttributeSets: AttributeSetList = None
        if "Relationships" in data:
            self.Relationships: RelationshipList = self._get_value(RelationshipList, "Relationships")
        else:
            self.Relationships: RelationshipList = None
        if "CompetitivePricing" in data:
            self.CompetitivePricing: CompetitivePricingType = self._get_value(
                CompetitivePricingType, "CompetitivePricing"
            )
        else:
            self.CompetitivePricing: CompetitivePricingType = None
        if "SalesRankings" in data:
            self.SalesRankings: SalesRankList = self._get_value(SalesRankList, "SalesRankings")
        else:
            self.SalesRankings: SalesRankList = None
        if "Offers" in data:
            self.Offers: OffersList = self._get_value(OffersList, "Offers")
        else:
            self.Offers: OffersList = None


class IdentifierType(__BaseDictObject):
    """
    Specifies the identifiers used to uniquely identify an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceASIN" in data:
            self.MarketplaceASIN: ASINIdentifier = self._get_value(ASINIdentifier, "MarketplaceASIN")
        else:
            self.MarketplaceASIN: ASINIdentifier = None
        if "SKUIdentifier" in data:
            self.SKUIdentifier: SellerSKUIdentifier = self._get_value(SellerSKUIdentifier, "SKUIdentifier")
        else:
            self.SKUIdentifier: SellerSKUIdentifier = None


class ASINIdentifier(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None


class SellerSKUIdentifier(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "SellerId" in data:
            self.SellerId: str = self._get_value(str, "SellerId")
        else:
            self.SellerId: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None


class CompetitivePricingType(__BaseDictObject):
    """
    Competitive pricing information for the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "CompetitivePrices" in data:
            self.CompetitivePrices: CompetitivePriceList = self._get_value(CompetitivePriceList, "CompetitivePrices")
        else:
            self.CompetitivePrices: CompetitivePriceList = None
        if "NumberOfOfferListings" in data:
            self.NumberOfOfferListings: NumberOfOfferListingsList = self._get_value(
                NumberOfOfferListingsList, "NumberOfOfferListings"
            )
        else:
            self.NumberOfOfferListings: NumberOfOfferListingsList = None
        if "TradeInValue" in data:
            self.TradeInValue: MoneyType = self._get_value(MoneyType, "TradeInValue")
        else:
            self.TradeInValue: MoneyType = None


class CompetitivePriceType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "CompetitivePriceId" in data:
            self.CompetitivePriceId: str = self._get_value(str, "CompetitivePriceId")
        else:
            self.CompetitivePriceId: str = None
        if "Price" in data:
            self.Price: PriceType = self._get_value(PriceType, "Price")
        else:
            self.Price: PriceType = None
        if "condition" in data:
            self.condition: str = self._get_value(str, "condition")
        else:
            self.condition: str = None
        if "subcondition" in data:
            self.subcondition: str = self._get_value(str, "subcondition")
        else:
            self.subcondition: str = None
        if "offerType" in data:
            self.offerType: OfferCustomerType = self._get_value(OfferCustomerType, "offerType")
        else:
            self.offerType: OfferCustomerType = None
        if "quantityTier" in data:
            self.quantityTier: int = self._get_value(int, "quantityTier")
        else:
            self.quantityTier: int = None
        if "quantityDiscountType" in data:
            self.quantityDiscountType: QuantityDiscountType = self._get_value(
                QuantityDiscountType, "quantityDiscountType"
            )
        else:
            self.quantityDiscountType: QuantityDiscountType = None
        if "sellerId" in data:
            self.sellerId: str = self._get_value(str, "sellerId")
        else:
            self.sellerId: str = None
        if "belongsToRequester" in data:
            self.belongsToRequester: bool = self._get_value(convert_bool, "belongsToRequester")
        else:
            self.belongsToRequester: bool = None


class OfferListingCountType(__BaseDictObject):
    """
    The number of offer listings with the specified condition.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Count" in data:
            self.Count: int = self._get_value(int, "Count")
        else:
            self.Count: int = None
        if "condition" in data:
            self.condition: str = self._get_value(str, "condition")
        else:
            self.condition: str = None


class MoneyType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "CurrencyCode" in data:
            self.CurrencyCode: str = self._get_value(str, "CurrencyCode")
        else:
            self.CurrencyCode: str = None
        if "Amount" in data:
            self.Amount: float = self._get_value(float, "Amount")
        else:
            self.Amount: float = None


class SalesRankType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "ProductCategoryId" in data:
            self.ProductCategoryId: str = self._get_value(str, "ProductCategoryId")
        else:
            self.ProductCategoryId: str = None
        if "Rank" in data:
            self.Rank: int = self._get_value(int, "Rank")
        else:
            self.Rank: int = None


class PriceType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = self._get_value(MoneyType, "LandedPrice")
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = self._get_value(MoneyType, "ListingPrice")
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = self._get_value(MoneyType, "Shipping")
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = self._get_value(Points, "Points")
        else:
            self.Points: Points = None


class OfferType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "offerType" in data:
            self.offerType: OfferCustomerType = self._get_value(OfferCustomerType, "offerType")
        else:
            self.offerType: OfferCustomerType = None
        if "BuyingPrice" in data:
            self.BuyingPrice: PriceType = self._get_value(PriceType, "BuyingPrice")
        else:
            self.BuyingPrice: PriceType = None
        if "RegularPrice" in data:
            self.RegularPrice: MoneyType = self._get_value(MoneyType, "RegularPrice")
        else:
            self.RegularPrice: MoneyType = None
        if "businessPrice" in data:
            self.businessPrice: MoneyType = self._get_value(MoneyType, "businessPrice")
        else:
            self.businessPrice: MoneyType = None
        if "quantityDiscountPrices" in data:
            self.quantityDiscountPrices: _List[QuantityDiscountPriceType] = [
                QuantityDiscountPriceType(datum) for datum in data["quantityDiscountPrices"]
            ]
        else:
            self.quantityDiscountPrices: _List[QuantityDiscountPriceType] = []
        if "FulfillmentChannel" in data:
            self.FulfillmentChannel: str = self._get_value(str, "FulfillmentChannel")
        else:
            self.FulfillmentChannel: str = None
        if "ItemCondition" in data:
            self.ItemCondition: str = self._get_value(str, "ItemCondition")
        else:
            self.ItemCondition: str = None
        if "ItemSubCondition" in data:
            self.ItemSubCondition: str = self._get_value(str, "ItemSubCondition")
        else:
            self.ItemSubCondition: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None


class QuantityDiscountPriceType(__BaseDictObject):
    """
    Contains pricing information that includes special pricing when buying in bulk.
    """

    def __init__(self, data):
        super().__init__(data)
        if "quantityTier" in data:
            self.quantityTier: int = self._get_value(int, "quantityTier")
        else:
            self.quantityTier: int = None
        if "quantityDiscountType" in data:
            self.quantityDiscountType: QuantityDiscountType = self._get_value(
                QuantityDiscountType, "quantityDiscountType"
            )
        else:
            self.quantityDiscountType: QuantityDiscountType = None
        if "listingPrice" in data:
            self.listingPrice: MoneyType = self._get_value(MoneyType, "listingPrice")
        else:
            self.listingPrice: MoneyType = None


class Points(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "PointsNumber" in data:
            self.PointsNumber: int = self._get_value(int, "PointsNumber")
        else:
            self.PointsNumber: int = None
        if "PointsMonetaryValue" in data:
            self.PointsMonetaryValue: MoneyType = self._get_value(MoneyType, "PointsMonetaryValue")
        else:
            self.PointsMonetaryValue: MoneyType = None


class ItemIdentifier(__BaseDictObject):
    """
    Information that identifies an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "ItemCondition" in data:
            self.ItemCondition: ConditionType = self._get_value(ConditionType, "ItemCondition")
        else:
            self.ItemCondition: ConditionType = None


class Summary(__BaseDictObject):
    """
    Contains price information about the product, including the LowestPrices and BuyBoxPrices, the ListPrice, the SuggestedLowerPricePlusShipping, and NumberOfOffers and NumberOfBuyBoxEligibleOffers.
    """

    def __init__(self, data):
        super().__init__(data)
        if "TotalOfferCount" in data:
            self.TotalOfferCount: int = self._get_value(int, "TotalOfferCount")
        else:
            self.TotalOfferCount: int = None
        if "NumberOfOffers" in data:
            self.NumberOfOffers: NumberOfOffers = self._get_value(NumberOfOffers, "NumberOfOffers")
        else:
            self.NumberOfOffers: NumberOfOffers = None
        if "LowestPrices" in data:
            self.LowestPrices: LowestPrices = self._get_value(LowestPrices, "LowestPrices")
        else:
            self.LowestPrices: LowestPrices = None
        if "BuyBoxPrices" in data:
            self.BuyBoxPrices: BuyBoxPrices = self._get_value(BuyBoxPrices, "BuyBoxPrices")
        else:
            self.BuyBoxPrices: BuyBoxPrices = None
        if "ListPrice" in data:
            self.ListPrice: MoneyType = self._get_value(MoneyType, "ListPrice")
        else:
            self.ListPrice: MoneyType = None
        if "CompetitivePriceThreshold" in data:
            self.CompetitivePriceThreshold: MoneyType = self._get_value(MoneyType, "CompetitivePriceThreshold")
        else:
            self.CompetitivePriceThreshold: MoneyType = None
        if "SuggestedLowerPricePlusShipping" in data:
            self.SuggestedLowerPricePlusShipping: MoneyType = self._get_value(
                MoneyType, "SuggestedLowerPricePlusShipping"
            )
        else:
            self.SuggestedLowerPricePlusShipping: MoneyType = None
        if "SalesRankings" in data:
            self.SalesRankings: SalesRankList = self._get_value(SalesRankList, "SalesRankings")
        else:
            self.SalesRankings: SalesRankList = None
        if "BuyBoxEligibleOffers" in data:
            self.BuyBoxEligibleOffers: BuyBoxEligibleOffers = self._get_value(
                BuyBoxEligibleOffers, "BuyBoxEligibleOffers"
            )
        else:
            self.BuyBoxEligibleOffers: BuyBoxEligibleOffers = None
        if "OffersAvailableTime" in data:
            self.OffersAvailableTime: str = self._get_value(str, "OffersAvailableTime")
        else:
            self.OffersAvailableTime: str = None


class OfferCountType(__BaseDictObject):
    """
    The total number of offers for the specified condition and fulfillment channel.
    """

    def __init__(self, data):
        super().__init__(data)
        if "condition" in data:
            self.condition: str = self._get_value(str, "condition")
        else:
            self.condition: str = None
        if "fulfillmentChannel" in data:
            self.fulfillmentChannel: FulfillmentChannelType = self._get_value(
                FulfillmentChannelType, "fulfillmentChannel"
            )
        else:
            self.fulfillmentChannel: FulfillmentChannelType = None
        if "OfferCount" in data:
            self.OfferCount: int = self._get_value(int, "OfferCount")
        else:
            self.OfferCount: int = None


class LowestPriceType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "condition" in data:
            self.condition: str = self._get_value(str, "condition")
        else:
            self.condition: str = None
        if "fulfillmentChannel" in data:
            self.fulfillmentChannel: str = self._get_value(str, "fulfillmentChannel")
        else:
            self.fulfillmentChannel: str = None
        if "offerType" in data:
            self.offerType: OfferCustomerType = self._get_value(OfferCustomerType, "offerType")
        else:
            self.offerType: OfferCustomerType = None
        if "quantityTier" in data:
            self.quantityTier: int = self._get_value(int, "quantityTier")
        else:
            self.quantityTier: int = None
        if "quantityDiscountType" in data:
            self.quantityDiscountType: QuantityDiscountType = self._get_value(
                QuantityDiscountType, "quantityDiscountType"
            )
        else:
            self.quantityDiscountType: QuantityDiscountType = None
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = self._get_value(MoneyType, "LandedPrice")
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = self._get_value(MoneyType, "ListingPrice")
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = self._get_value(MoneyType, "Shipping")
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = self._get_value(Points, "Points")
        else:
            self.Points: Points = None


class BuyBoxPriceType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "condition" in data:
            self.condition: str = self._get_value(str, "condition")
        else:
            self.condition: str = None
        if "offerType" in data:
            self.offerType: OfferCustomerType = self._get_value(OfferCustomerType, "offerType")
        else:
            self.offerType: OfferCustomerType = None
        if "quantityTier" in data:
            self.quantityTier: int = self._get_value(int, "quantityTier")
        else:
            self.quantityTier: int = None
        if "quantityDiscountType" in data:
            self.quantityDiscountType: QuantityDiscountType = self._get_value(
                QuantityDiscountType, "quantityDiscountType"
            )
        else:
            self.quantityDiscountType: QuantityDiscountType = None
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = self._get_value(MoneyType, "LandedPrice")
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = self._get_value(MoneyType, "ListingPrice")
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = self._get_value(MoneyType, "Shipping")
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = self._get_value(Points, "Points")
        else:
            self.Points: Points = None
        if "sellerId" in data:
            self.sellerId: str = self._get_value(str, "sellerId")
        else:
            self.sellerId: str = None


class OfferDetail(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MyOffer" in data:
            self.MyOffer: bool = self._get_value(convert_bool, "MyOffer")
        else:
            self.MyOffer: bool = None
        if "offerType" in data:
            self.offerType: OfferCustomerType = self._get_value(OfferCustomerType, "offerType")
        else:
            self.offerType: OfferCustomerType = None
        if "SubCondition" in data:
            self.SubCondition: str = self._get_value(str, "SubCondition")
        else:
            self.SubCondition: str = None
        if "SellerId" in data:
            self.SellerId: str = self._get_value(str, "SellerId")
        else:
            self.SellerId: str = None
        if "ConditionNotes" in data:
            self.ConditionNotes: str = self._get_value(str, "ConditionNotes")
        else:
            self.ConditionNotes: str = None
        if "SellerFeedbackRating" in data:
            self.SellerFeedbackRating: SellerFeedbackType = self._get_value(SellerFeedbackType, "SellerFeedbackRating")
        else:
            self.SellerFeedbackRating: SellerFeedbackType = None
        if "ShippingTime" in data:
            self.ShippingTime: DetailedShippingTimeType = self._get_value(DetailedShippingTimeType, "ShippingTime")
        else:
            self.ShippingTime: DetailedShippingTimeType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = self._get_value(MoneyType, "ListingPrice")
        else:
            self.ListingPrice: MoneyType = None
        if "quantityDiscountPrices" in data:
            self.quantityDiscountPrices: _List[QuantityDiscountPriceType] = [
                QuantityDiscountPriceType(datum) for datum in data["quantityDiscountPrices"]
            ]
        else:
            self.quantityDiscountPrices: _List[QuantityDiscountPriceType] = []
        if "Points" in data:
            self.Points: Points = self._get_value(Points, "Points")
        else:
            self.Points: Points = None
        if "Shipping" in data:
            self.Shipping: MoneyType = self._get_value(MoneyType, "Shipping")
        else:
            self.Shipping: MoneyType = None
        if "ShipsFrom" in data:
            self.ShipsFrom: ShipsFromType = self._get_value(ShipsFromType, "ShipsFrom")
        else:
            self.ShipsFrom: ShipsFromType = None
        if "IsFulfilledByAmazon" in data:
            self.IsFulfilledByAmazon: bool = self._get_value(convert_bool, "IsFulfilledByAmazon")
        else:
            self.IsFulfilledByAmazon: bool = None
        if "PrimeInformation" in data:
            self.PrimeInformation: PrimeInformationType = self._get_value(PrimeInformationType, "PrimeInformation")
        else:
            self.PrimeInformation: PrimeInformationType = None
        if "IsBuyBoxWinner" in data:
            self.IsBuyBoxWinner: bool = self._get_value(convert_bool, "IsBuyBoxWinner")
        else:
            self.IsBuyBoxWinner: bool = None
        if "IsFeaturedMerchant" in data:
            self.IsFeaturedMerchant: bool = self._get_value(convert_bool, "IsFeaturedMerchant")
        else:
            self.IsFeaturedMerchant: bool = None


class PrimeInformationType(__BaseDictObject):
    """
    Amazon Prime information.
    """

    def __init__(self, data):
        super().__init__(data)
        if "IsPrime" in data:
            self.IsPrime: bool = self._get_value(convert_bool, "IsPrime")
        else:
            self.IsPrime: bool = None
        if "IsNationalPrime" in data:
            self.IsNationalPrime: bool = self._get_value(convert_bool, "IsNationalPrime")
        else:
            self.IsNationalPrime: bool = None


class SellerFeedbackType(__BaseDictObject):
    """
    Information about the seller's feedback, including the percentage of positive feedback, and the total number of ratings received.
    """

    def __init__(self, data):
        super().__init__(data)
        if "SellerPositiveFeedbackRating" in data:
            self.SellerPositiveFeedbackRating: float = self._get_value(float, "SellerPositiveFeedbackRating")
        else:
            self.SellerPositiveFeedbackRating: float = None
        if "FeedbackCount" in data:
            self.FeedbackCount: int = self._get_value(int, "FeedbackCount")
        else:
            self.FeedbackCount: int = None


class DetailedShippingTimeType(__BaseDictObject):
    """
    The time range in which an item will likely be shipped once an order has been placed.
    """

    def __init__(self, data):
        super().__init__(data)
        if "minimumHours" in data:
            self.minimumHours: int = self._get_value(int, "minimumHours")
        else:
            self.minimumHours: int = None
        if "maximumHours" in data:
            self.maximumHours: int = self._get_value(int, "maximumHours")
        else:
            self.maximumHours: int = None
        if "availableDate" in data:
            self.availableDate: str = self._get_value(str, "availableDate")
        else:
            self.availableDate: str = None
        if "availabilityType" in data:
            self.availabilityType: str = self._get_value(str, "availabilityType")
        else:
            self.availabilityType: str = None


class ShipsFromType(__BaseDictObject):
    """
    The state and country from where the item is shipped.
    """

    def __init__(self, data):
        super().__init__(data)
        if "State" in data:
            self.State: str = self._get_value(str, "State")
        else:
            self.State: str = None
        if "Country" in data:
            self.Country: str = self._get_value(str, "Country")
        else:
            self.Country: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class PriceList(list, _List["Price"]):
    """ """

    def __init__(self, data):
        super().__init__([Price(datum) for datum in data])
        self.data = data


class AttributeSetList(list, _List["dict"]):
    """
    A list of product attributes if they are applicable to the product that is returned.
    """

    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class RelationshipList(list, _List["dict"]):
    """
    A list that contains product variation information, if applicable.
    """

    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class CompetitivePriceList(list, _List["CompetitivePriceType"]):
    """
    A list of competitive pricing information.
    """

    def __init__(self, data):
        super().__init__([CompetitivePriceType(datum) for datum in data])
        self.data = data


class NumberOfOfferListingsList(list, _List["OfferListingCountType"]):
    """
    The number of active offer listings for the item that was submitted. The listing count is returned by condition, one for each listing condition value that is returned.
    """

    def __init__(self, data):
        super().__init__([OfferListingCountType(datum) for datum in data])
        self.data = data


class SalesRankList(list, _List["SalesRankType"]):
    """
    A list of sales rank information for the item, by category.
    """

    def __init__(self, data):
        super().__init__([SalesRankType(datum) for datum in data])
        self.data = data


class OffersList(list, _List["OfferType"]):
    """
    A list of offers.
    """

    def __init__(self, data):
        super().__init__([OfferType(datum) for datum in data])
        self.data = data


class BuyBoxEligibleOffers(list, _List["OfferCountType"]):
    """ """

    def __init__(self, data):
        super().__init__([OfferCountType(datum) for datum in data])
        self.data = data


class BuyBoxPrices(list, _List["BuyBoxPriceType"]):
    """ """

    def __init__(self, data):
        super().__init__([BuyBoxPriceType(datum) for datum in data])
        self.data = data


class LowestPrices(list, _List["LowestPriceType"]):
    """ """

    def __init__(self, data):
        super().__init__([LowestPriceType(datum) for datum in data])
        self.data = data


class NumberOfOffers(list, _List["OfferCountType"]):
    """ """

    def __init__(self, data):
        super().__init__([OfferCountType(datum) for datum in data])
        self.data = data


class OfferDetailList(list, _List["OfferDetail"]):
    """ """

    def __init__(self, data):
        super().__init__([OfferDetail(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class OfferCustomerType(str):
    """ """


class QuantityDiscountType(str):
    """ """


class ConditionType(str):
    """
    Indicates the condition of the item. Possible values: New, Used, Collectible, Refurbished, Club.
    """


class FulfillmentChannelType(str):
    """
    Indicates whether the item is fulfilled by Amazon or by the seller (merchant).
    """


class ProductPricingV0Client(__BaseClient):
    def getPricing(
        self,
        MarketplaceId: str,
        ItemType: str,
        Asins: _List[str] = None,
        Skus: _List[str] = None,
        ItemCondition: str = None,
        OfferType: str = None,
    ):
        """
                Returns pricing information for a seller's offer listings based on seller SKU or ASIN.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/products/pricing/v0/price"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if Asins is not None:
            params["Asins"] = ",".join(map(str, Asins))
        if Skus is not None:
            params["Skus"] = ",".join(map(str, Skus))
        if ItemType is not None:
            params["ItemType"] = ItemType
        if ItemCondition is not None:
            params["ItemCondition"] = ItemCondition
        if OfferType is not None:
            params["OfferType"] = OfferType
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetPricingResponse,
            400: GetPricingResponse,
            401: GetPricingResponse,
            403: GetPricingResponse,
            404: GetPricingResponse,
            429: GetPricingResponse,
            500: GetPricingResponse,
            503: GetPricingResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getCompetitivePricing(
        self,
        MarketplaceId: str,
        ItemType: str,
        Asins: _List[str] = None,
        Skus: _List[str] = None,
        CustomerType: str = None,
    ):
        """
                Returns competitive pricing information for a seller's offer listings based on seller SKU or ASIN.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/products/pricing/v0/competitivePrice"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if Asins is not None:
            params["Asins"] = ",".join(map(str, Asins))
        if Skus is not None:
            params["Skus"] = ",".join(map(str, Skus))
        if ItemType is not None:
            params["ItemType"] = ItemType
        if CustomerType is not None:
            params["CustomerType"] = CustomerType
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetPricingResponse,
            400: GetPricingResponse,
            401: GetPricingResponse,
            403: GetPricingResponse,
            404: GetPricingResponse,
            429: GetPricingResponse,
            500: GetPricingResponse,
            503: GetPricingResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getListingOffers(
        self,
        SellerSKU: str,
        MarketplaceId: str,
        ItemCondition: str,
        CustomerType: str = None,
    ):
        """
                Returns the lowest priced offers for a single SKU listing.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 5 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/products/pricing/v0/listings/{SellerSKU}/offers"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if ItemCondition is not None:
            params["ItemCondition"] = ItemCondition
        if CustomerType is not None:
            params["CustomerType"] = CustomerType
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOffersResponse,
            400: GetOffersResponse,
            401: GetOffersResponse,
            403: GetOffersResponse,
            404: GetOffersResponse,
            429: GetOffersResponse,
            500: GetOffersResponse,
            503: GetOffersResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getItemOffers(
        self,
        Asin: str,
        MarketplaceId: str,
        ItemCondition: str,
        CustomerType: str = None,
    ):
        """
                Returns the lowest priced offers for a single item based on ASIN.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 5 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/products/pricing/v0/items/{Asin}/offers"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if ItemCondition is not None:
            params["ItemCondition"] = ItemCondition
        if CustomerType is not None:
            params["CustomerType"] = CustomerType
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOffersResponse,
            400: GetOffersResponse,
            401: GetOffersResponse,
            403: GetOffersResponse,
            404: GetOffersResponse,
            429: GetOffersResponse,
            500: GetOffersResponse,
            503: GetOffersResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class SubmitShippingLabelsRequest(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "shippingLabelRequests" in data:
            self.shippingLabelRequests: _List[ShippingLabelRequest] = [
                ShippingLabelRequest(datum) for datum in data["shippingLabelRequests"]
            ]
        else:
            self.shippingLabelRequests: _List[ShippingLabelRequest] = []


class ShippingLabelRequest(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "sellingParty" in data:
            self.sellingParty: PartyIdentification = self._get_value(PartyIdentification, "sellingParty")
        else:
            self.sellingParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None
        if "containers" in data:
            self.containers: _List[Container] = [Container(datum) for datum in data["containers"]]
        else:
            self.containers: _List[Container] = []


class Item(__BaseDictObject):
    """
    Details of the item being shipped.
    """

    def __init__(self, data):
        super().__init__(data)
        if "itemSequenceNumber" in data:
            self.itemSequenceNumber: int = self._get_value(int, "itemSequenceNumber")
        else:
            self.itemSequenceNumber: int = None
        if "buyerProductIdentifier" in data:
            self.buyerProductIdentifier: str = self._get_value(str, "buyerProductIdentifier")
        else:
            self.buyerProductIdentifier: str = None
        if "vendorProductIdentifier" in data:
            self.vendorProductIdentifier: str = self._get_value(str, "vendorProductIdentifier")
        else:
            self.vendorProductIdentifier: str = None
        if "shippedQuantity" in data:
            self.shippedQuantity: ItemQuantity = self._get_value(ItemQuantity, "shippedQuantity")
        else:
            self.shippedQuantity: ItemQuantity = None


class PackedItem(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "itemSequenceNumber" in data:
            self.itemSequenceNumber: int = self._get_value(int, "itemSequenceNumber")
        else:
            self.itemSequenceNumber: int = None
        if "buyerProductIdentifier" in data:
            self.buyerProductIdentifier: str = self._get_value(str, "buyerProductIdentifier")
        else:
            self.buyerProductIdentifier: str = None
        if "vendorProductIdentifier" in data:
            self.vendorProductIdentifier: str = self._get_value(str, "vendorProductIdentifier")
        else:
            self.vendorProductIdentifier: str = None
        if "packedQuantity" in data:
            self.packedQuantity: ItemQuantity = self._get_value(ItemQuantity, "packedQuantity")
        else:
            self.packedQuantity: ItemQuantity = None


class PartyIdentification(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "partyId" in data:
            self.partyId: str = self._get_value(str, "partyId")
        else:
            self.partyId: str = None
        if "address" in data:
            self.address: Address = self._get_value(Address, "address")
        else:
            self.address: Address = None
        if "taxRegistrationDetails" in data:
            self.taxRegistrationDetails: _List[TaxRegistrationDetails] = [
                TaxRegistrationDetails(datum) for datum in data["taxRegistrationDetails"]
            ]
        else:
            self.taxRegistrationDetails: _List[TaxRegistrationDetails] = []


class TaxRegistrationDetails(__BaseDictObject):
    """
    Tax registration details of the entity.
    """

    def __init__(self, data):
        super().__init__(data)
        if "taxRegistrationType" in data:
            self.taxRegistrationType: str = self._get_value(str, "taxRegistrationType")
        else:
            self.taxRegistrationType: str = None
        if "taxRegistrationNumber" in data:
            self.taxRegistrationNumber: str = self._get_value(str, "taxRegistrationNumber")
        else:
            self.taxRegistrationNumber: str = None
        if "taxRegistrationAddress" in data:
            self.taxRegistrationAddress: Address = self._get_value(Address, "taxRegistrationAddress")
        else:
            self.taxRegistrationAddress: Address = None
        if "taxRegistrationMessages" in data:
            self.taxRegistrationMessages: str = self._get_value(str, "taxRegistrationMessages")
        else:
            self.taxRegistrationMessages: str = None


class Address(__BaseDictObject):
    """
    Address of the party.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "addressLine1" in data:
            self.addressLine1: str = self._get_value(str, "addressLine1")
        else:
            self.addressLine1: str = None
        if "addressLine2" in data:
            self.addressLine2: str = self._get_value(str, "addressLine2")
        else:
            self.addressLine2: str = None
        if "addressLine3" in data:
            self.addressLine3: str = self._get_value(str, "addressLine3")
        else:
            self.addressLine3: str = None
        if "city" in data:
            self.city: str = self._get_value(str, "city")
        else:
            self.city: str = None
        if "county" in data:
            self.county: str = self._get_value(str, "county")
        else:
            self.county: str = None
        if "district" in data:
            self.district: str = self._get_value(str, "district")
        else:
            self.district: str = None
        if "stateOrRegion" in data:
            self.stateOrRegion: str = self._get_value(str, "stateOrRegion")
        else:
            self.stateOrRegion: str = None
        if "postalCode" in data:
            self.postalCode: str = self._get_value(str, "postalCode")
        else:
            self.postalCode: str = None
        if "countryCode" in data:
            self.countryCode: str = self._get_value(str, "countryCode")
        else:
            self.countryCode: str = None
        if "phone" in data:
            self.phone: str = self._get_value(str, "phone")
        else:
            self.phone: str = None


class Dimensions(__BaseDictObject):
    """
    Physical dimensional measurements of a container.
    """

    def __init__(self, data):
        super().__init__(data)
        if "length" in data:
            self.length: Decimal = self._get_value(Decimal, "length")
        else:
            self.length: Decimal = None
        if "width" in data:
            self.width: Decimal = self._get_value(Decimal, "width")
        else:
            self.width: Decimal = None
        if "height" in data:
            self.height: Decimal = self._get_value(Decimal, "height")
        else:
            self.height: Decimal = None
        if "unitOfMeasure" in data:
            self.unitOfMeasure: str = self._get_value(str, "unitOfMeasure")
        else:
            self.unitOfMeasure: str = None


class Weight(__BaseDictObject):
    """
    The weight.
    """

    def __init__(self, data):
        super().__init__(data)
        if "unitOfMeasure" in data:
            self.unitOfMeasure: str = self._get_value(str, "unitOfMeasure")
        else:
            self.unitOfMeasure: str = None
        if "value" in data:
            self.value: Decimal = self._get_value(Decimal, "value")
        else:
            self.value: Decimal = None


class ItemQuantity(__BaseDictObject):
    """
    Details of item quantity.
    """

    def __init__(self, data):
        super().__init__(data)
        if "amount" in data:
            self.amount: int = self._get_value(int, "amount")
        else:
            self.amount: int = None
        if "unitOfMeasure" in data:
            self.unitOfMeasure: str = self._get_value(str, "unitOfMeasure")
        else:
            self.unitOfMeasure: str = None


class GetShippingLabelListResponse(__BaseDictObject):
    """
    The response schema for the getShippingLabels operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ShippingLabelList = self._get_value(ShippingLabelList, "payload")
        else:
            self.payload: ShippingLabelList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetShippingLabelResponse(__BaseDictObject):
    """
    The response schema for the getShippingLabel operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ShippingLabel = self._get_value(ShippingLabel, "payload")
        else:
            self.payload: ShippingLabel = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class ShippingLabelList(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "pagination" in data:
            self.pagination: Pagination = self._get_value(Pagination, "pagination")
        else:
            self.pagination: Pagination = None
        if "shippingLabels" in data:
            self.shippingLabels: _List[ShippingLabel] = [ShippingLabel(datum) for datum in data["shippingLabels"]]
        else:
            self.shippingLabels: _List[ShippingLabel] = []


class LabelData(__BaseDictObject):
    """
    Details of the shipment label.
    """

    def __init__(self, data):
        super().__init__(data)
        if "packageIdentifier" in data:
            self.packageIdentifier: str = self._get_value(str, "packageIdentifier")
        else:
            self.packageIdentifier: str = None
        if "trackingNumber" in data:
            self.trackingNumber: str = self._get_value(str, "trackingNumber")
        else:
            self.trackingNumber: str = None
        if "shipMethod" in data:
            self.shipMethod: str = self._get_value(str, "shipMethod")
        else:
            self.shipMethod: str = None
        if "shipMethodName" in data:
            self.shipMethodName: str = self._get_value(str, "shipMethodName")
        else:
            self.shipMethodName: str = None
        if "content" in data:
            self.content: str = self._get_value(str, "content")
        else:
            self.content: str = None


class ShippingLabel(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "sellingParty" in data:
            self.sellingParty: PartyIdentification = self._get_value(PartyIdentification, "sellingParty")
        else:
            self.sellingParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None
        if "labelFormat" in data:
            self.labelFormat: str = self._get_value(str, "labelFormat")
        else:
            self.labelFormat: str = None
        if "labelData" in data:
            self.labelData: _List[LabelData] = [LabelData(datum) for datum in data["labelData"]]
        else:
            self.labelData: _List[LabelData] = []


class SubmitShippingLabelsResponse(__BaseDictObject):
    """
    The response schema for the submitShippingLabelRequest operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: TransactionReference = self._get_value(TransactionReference, "payload")
        else:
            self.payload: TransactionReference = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetCustomerInvoicesResponse(__BaseDictObject):
    """
    The response schema for the getCustomerInvoices operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: CustomerInvoiceList = self._get_value(CustomerInvoiceList, "payload")
        else:
            self.payload: CustomerInvoiceList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetCustomerInvoiceResponse(__BaseDictObject):
    """
    The response schema for the getCustomerInvoice operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: CustomerInvoice = self._get_value(CustomerInvoice, "payload")
        else:
            self.payload: CustomerInvoice = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class CustomerInvoiceList(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "pagination" in data:
            self.pagination: Pagination = self._get_value(Pagination, "pagination")
        else:
            self.pagination: Pagination = None
        if "customerInvoices" in data:
            self.customerInvoices: _List[CustomerInvoice] = [
                CustomerInvoice(datum) for datum in data["customerInvoices"]
            ]
        else:
            self.customerInvoices: _List[CustomerInvoice] = []


class Pagination(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "nextToken" in data:
            self.nextToken: str = self._get_value(str, "nextToken")
        else:
            self.nextToken: str = None


class CustomerInvoice(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "content" in data:
            self.content: str = self._get_value(str, "content")
        else:
            self.content: str = None


class TransactionReference(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class Container(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "containerType" in data:
            self.containerType: str = self._get_value(str, "containerType")
        else:
            self.containerType: str = None
        if "containerIdentifier" in data:
            self.containerIdentifier: str = self._get_value(str, "containerIdentifier")
        else:
            self.containerIdentifier: str = None
        if "trackingNumber" in data:
            self.trackingNumber: str = self._get_value(str, "trackingNumber")
        else:
            self.trackingNumber: str = None
        if "manifestId" in data:
            self.manifestId: str = self._get_value(str, "manifestId")
        else:
            self.manifestId: str = None
        if "manifestDate" in data:
            self.manifestDate: str = self._get_value(str, "manifestDate")
        else:
            self.manifestDate: str = None
        if "shipMethod" in data:
            self.shipMethod: str = self._get_value(str, "shipMethod")
        else:
            self.shipMethod: str = None
        if "scacCode" in data:
            self.scacCode: str = self._get_value(str, "scacCode")
        else:
            self.scacCode: str = None
        if "carrier" in data:
            self.carrier: str = self._get_value(str, "carrier")
        else:
            self.carrier: str = None
        if "containerSequenceNumber" in data:
            self.containerSequenceNumber: int = self._get_value(int, "containerSequenceNumber")
        else:
            self.containerSequenceNumber: int = None
        if "dimensions" in data:
            self.dimensions: Dimensions = self._get_value(Dimensions, "dimensions")
        else:
            self.dimensions: Dimensions = None
        if "weight" in data:
            self.weight: Weight = self._get_value(Weight, "weight")
        else:
            self.weight: Weight = None
        if "packedItems" in data:
            self.packedItems: _List[PackedItem] = [PackedItem(datum) for datum in data["packedItems"]]
        else:
            self.packedItems: _List[PackedItem] = []


class Decimal(str):
    """
    A decimal number with no loss of precision. Useful when precision loss is unacceptable, as with currencies. Follows RFC7159 for number representation.  <br>**Pattern** : `^-?(0|([1-9]\\d*))(\\.\\d+)?([eE][+-]?\\d+)?$`.
    """


class VendorDirectFulfillmentShipping20211228Client(__BaseClient):
    def getShippingLabels(
        self,
        createdAfter: str,
        createdBefore: str,
        shipFromPartyId: str = None,
        limit: int = None,
        sortOrder: str = None,
        nextToken: str = None,
    ):
        """
                Returns a list of shipping labels created during the time frame that you specify. You define that time frame using the createdAfter and createdBefore parameters. You must use both of these parameters. The date range to search must not be more than 7 days.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 10 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/vendor/directFulfillment/shipping/2021-12-28/shippingLabels"
        params = {}
        if shipFromPartyId is not None:
            params["shipFromPartyId"] = shipFromPartyId
        if limit is not None:
            params["limit"] = limit
        if createdAfter is not None:
            params["createdAfter"] = createdAfter
        if createdBefore is not None:
            params["createdBefore"] = createdBefore
        if sortOrder is not None:
            params["sortOrder"] = sortOrder
        if nextToken is not None:
            params["nextToken"] = nextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ShippingLabelList,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def submitShippingLabelRequest(
        self,
        data: SubmitShippingLabelsRequest,
    ):
        """
                Creates a shipping label for a purchase order and returns a transactionId for reference.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 10 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/vendor/directFulfillment/shipping/2021-12-28/shippingLabels"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: TransactionReference,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getShippingLabel(
        self,
        purchaseOrderNumber: str,
    ):
        """
                Returns a shipping label for the purchaseOrderNumber that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 10 | 10 |
        The `x-amzn-RateLimit-Limit` response header returns the usage plan rate limits that were applied to the requested operation, when available. The table above indicates the default rate and burst values for this operation. Selling partners whose business demands require higher throughput may see higher rate and burst values then those shown here. For more information, see [Usage Plans and Rate Limits in the Selling Partner API](doc:usage-plans-and-rate-limits-in-the-sp-api).
        """
        url = f"/vendor/directFulfillment/shipping/2021-12-28/shippingLabels/{purchaseOrderNumber}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ShippingLabel,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))
